"""
Test the helper functions in aasrp/helper/user.py
"""

# Django
from django.contrib.auth.models import User
from django.test import TestCase

# AA SRP
from aasrp.helper.user import get_user_settings
from aasrp.models import UserSetting


class TestGetUserSettings(TestCase):
    """
    Test the get_user_settings function
    """

    def test_get_user_settings_creates_new_settings_for_new_user(self):
        """
        Test that get_user_settings creates new settings for a new user

        :return:
        :rtype:
        """

        user = User.objects.create(username="newuser")
        user_settings = get_user_settings(user)

        self.assertIsInstance(user_settings, UserSetting)
        self.assertEqual(user_settings.user, user)

    def test_get_user_settings_returns_existing_settings_for_existing_user(self):
        """
        Test that get_user_settings returns existing settings for an existing user

        :return:
        :rtype:
        """

        user = User.objects.create(username="existinguser")
        existing_settings = UserSetting.objects.create(user=user)
        user_settings = get_user_settings(user)

        self.assertEqual(user_settings, existing_settings)

    def test_get_user_settings_handles_multiple_calls_for_same_user(self):
        """
        Test that get_user_settings handles multiple calls for the same user

        :return:
        :rtype:
        """

        user = User.objects.create(username="repeateduser")
        first_call_settings = get_user_settings(user)
        second_call_settings = get_user_settings(user)

        self.assertEqual(first_call_settings, second_call_settings)



# Django
from django.test import TestCase

# AA SRP
from aasrp.models import FleetType


class TestFleetType(TestCase):
    """
    Tests for FleetType model
    """

    def test_model_string_name(self):
        """
        Test FleetType model string names

        :return:
        :rtype:
        """

        expected_model_name = "Test Fleet Type"
        fleet_type = FleetType(name=expected_model_name)
        fleet_type.save()

        self.assertEqual(first=str(fleet_type), second=expected_model_name)

    def test_model_verbose_names(self):
        """
        Test FleetType model verbose names

        :return:
        :rtype:
        """

        self.assertEqual(first=FleetType._meta.verbose_name, second="Fleet type")
        self.assertEqual(
            first=FleetType._meta.verbose_name_plural, second="Fleet types"
        )

    def test_add_new_fleet_type(self):
        """
        Test adding a new FleetType

        :return:
        :rtype:
        """

        # given
        fleet_type = FleetType(name="Test Fleet Type")

        # when
        fleet_type.save()

        # then
        self.assertEqual(first=fleet_type.name, second="Test Fleet Type")



"""
Django admin declarations
"""

# Third Party
from solo.admin import SingletonModelAdmin

# Django
from django.contrib import admin, messages
from django.db.models import QuerySet
from django.http import HttpRequest
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext

# Alliance Auth
from allianceauth.framework.api.user import get_main_character_name_from_user

# AA SRP
from aasrp.form import SettingAdminForm
from aasrp.models import FleetType, RequestComment, Setting, SrpLink, SrpRequest


@admin.register(SrpLink)
class SrpLinkAdmin(admin.ModelAdmin):
    """
    SrpLinkAdmin
    """

    list_display = (
        "srp_code",
        "srp_name",
        "fleet_time",
        "_creator",
        "srp_status",
        "fleet_doctrine",
    )
    ordering = ("fleet_time",)
    list_filter = ("creator", "srp_status", "fleet_doctrine")
    search_fields = ("srp_code", "fleet_doctrine", "srp_name")

    @classmethod
    @admin.display(description=_("Creator"), ordering="creator")
    def _creator(cls, obj: SrpLink) -> str:
        """
        Display the creator name

        :param obj: The SrpLink object
        :type obj: SrpLink
        :return: The name of the creator
        :rtype: str
        """

        return get_main_character_name_from_user(obj.creator)


@admin.register(SrpRequest)
class SrpRequestAdmin(admin.ModelAdmin):
    """
    SrpRequestAdmin
    """

    list_display = (
        "request_code",
        "_requestor",
        "character",
        "srp_link",
        "_srp_code",
        "post_time",
        "ship",
        "loss_amount",
        "payout_amount",
        "killboard_link",
        "request_status",
    )
    ordering = ("post_time",)
    list_filter = ("creator", "character", "request_status")
    search_fields = (
        "request_code",
        "character__character_name",
        "ship__name",
        "srp_link__srp_name",
        "srp_link__srp_code",
    )

    @classmethod
    @admin.display(description=_("Requestor"), ordering="creator")
    def _requestor(cls, obj: SrpRequest) -> str:
        """
        Display the requestor name

        :param obj: The SrpRequest object
        :type obj: SrpRequest
        :return: The name of the requestor
        :rtype: str
        """

        return get_main_character_name_from_user(obj.creator)

    @admin.display(description=_("SRP code"), ordering="srp_link__srp_code")
    def _srp_code(self, obj: SrpRequest) -> str:
        """
        Display the SRP code

        :param obj: The SrpRequest object
        :type obj: SrpRequest
        :return: The SRP code associated with the request
        :rtype: str
        """

        return obj.srp_link.srp_code


@admin.register(RequestComment)
class RequestCommentAdmin(admin.ModelAdmin):
    """
    RequestCommentAdmin
    """

    list_display = (
        "_request_code",
        "_srp_code",
        "_requestor",
        "_character",
        "comment_type",
    )
    ordering = ("srp_request",)
    list_filter = ("comment_type",)
    search_fields = (
        "srp_request__request_code",
        "srp_request__character__character_name",
        "srp_request__ship__name",
        "srp_request__srp_link__srp_code",
    )

    @admin.display(description=_("SRP code"), ordering="srp_request__srp_code")
    def _srp_code(self, obj: RequestComment) -> str:
        """
        Display the SRP code

        :param obj: The RequestComment object
        :type obj: RequestComment
        :return: The SRP code associated with the request
        :rtype: str
        """

        return obj.srp_request.srp_link.srp_code

    @admin.display(description=_("Request code"), ordering="srp_request__request_code")
    def _request_code(self, obj: RequestComment) -> str:
        """
        Display the request code

        :param obj: The RequestComment object
        :type obj: RequestComment
        :return: The request code associated with the comment
        :rtype: str
        """

        return obj.srp_request.request_code

    @admin.display(description=_("Requestor"), ordering="srp_request__creator")
    def _requestor(self, obj: RequestComment) -> str:
        """
        Display the requestor name

        :param obj: The RequestComment object
        :type obj: RequestComment
        :return: The name of the requestor
        :rtype: str
        """

        return get_main_character_name_from_user(obj.srp_request.creator)

    @admin.display(description=_("Character"), ordering="srp_request__character")
    def _character(self, obj: RequestComment) -> str:
        """
        Display the character name the request is for

        :param obj: The RequestComment object
        :type obj: RequestComment
        :return: The name of the character the request is for
        :rtype: str
        """

        return obj.srp_request.character.character_name


@admin.register(FleetType)
class FleetTypeAdmin(admin.ModelAdmin):
    """
    FleetTypeAdmin
    """

    list_display = ("name", "is_enabled")
    list_filter = ("is_enabled",)
    ordering = ("name",)
    search_fields = ("name",)
    actions = ("activate", "deactivate")

    @admin.action(description=_("Activate selected %(verbose_name_plural)s"))
    def activate(self, request: HttpRequest, queryset: QuerySet[FleetType]) -> None:
        """
        Mark fleet type as active

        :param request: The request object
        :type request: HttpRequest
        :param queryset: The queryset of FleetType objects to activate
        :type queryset: QuerySet[FleetType]
        :return: None
        :rtype: NoneType
        """

        notifications_count = 0
        failed = 0

        for obj in queryset:
            try:
                obj.is_enabled = True
                obj.save()

                notifications_count += 1
            except Exception:  # pylint: disable=broad-exception-caught
                failed += 1

        if failed:
            messages.error(
                request=request,
                message=ngettext(
                    singular="Failed to activate {failed} fleet type",
                    plural="Failed to activate {failed} fleet types",
                    number=failed,
                ).format(failed=failed),
            )

        if queryset.count() - failed > 0:
            messages.success(
                request=request,
                message=ngettext(
                    singular="Activated {notifications_count} fleet type",
                    plural="Activated {notifications_count} fleet types",
                    number=notifications_count,
                ).format(notifications_count=notifications_count),
            )

    @admin.action(description=_("Deactivate selected %(verbose_name_plural)s"))
    def deactivate(self, request: HttpRequest, queryset: QuerySet[FleetType]) -> None:
        """
        Mark fleet type as inactive

        :param request: The request object
        :type request: HttpRequest
        :param queryset: The queryset of FleetType objects to deactivate
        :type queryset: QuerySet[FleetType]
        :return: None
        :rtype: NoneType
        """

        notifications_count = 0
        failed = 0

        for obj in queryset:
            try:
                obj.is_enabled = False
                obj.save()

                notifications_count += 1
            except Exception:  # pylint: disable=broad-exception-caught
                failed += 1

        if failed:
            messages.error(
                request=request,
                message=ngettext(
                    singular="Failed to deactivate {failed} fleet type",
                    plural="Failed to deactivate {failed} fleet types",
                    number=failed,
                ).format(failed=failed),
            )

        if queryset.count() - failed > 0:
            messages.success(
                request=request,
                message=ngettext(
                    singular="Deactivated {notifications_count} fleet type",
                    plural="Deactivated {notifications_count} fleet types",
                    number=notifications_count,
                ).format(notifications_count=notifications_count),
            )


@admin.register(Setting)
class SettingAdmin(SingletonModelAdmin):
    """
    Setting Admin
    """

    form = SettingAdminForm



"""
Providers
"""

# Alliance Auth
from esi.clients import EsiClientProvider

# AA SRP
from aasrp import __app_name_useragent__, __github_url__, __version__

# ESI client
esi = EsiClientProvider(
    ua_appname=__app_name_useragent__, ua_version=__version__, ua_url=__github_url__
)



"""
Helper functions for number formatting and localization.
"""

# Django
from django.template.defaultfilters import floatformat


def l10n_number_format(value: float, number_of_decimals: int = 0) -> str:
    """
    Localize a number for display in the template.

    :param value: The number to be localized.
    :type value: float
    :param number_of_decimals: The number of decimal places to display.
    :type number_of_decimals: int
    :return: The localized number as a string.
    :rtype: str
    """

    return floatformat(text=value, arg=f"{number_of_decimals}g")



"""
Eve images helper
"""

# Alliance Auth
from allianceauth.eveonline.evelinks.eveimageserver import (
    character_portrait_url,
    type_render_url,
)
from allianceauth.eveonline.models import EveCharacter


def get_character_portrait_from_evecharacter(
    character: EveCharacter, size: int = 32, as_html: bool = False
) -> str:
    """
    Get the character portrait from EveCharacter model

    :param character:
    :type character:
    :param size:
    :type size:
    :param as_html:
    :type as_html:
    :return:
    :rtype:
    """

    portrait_url = character_portrait_url(
        character_id=character.character_id, size=size
    )

    if as_html:
        return (
            '<img class="aasrp-character-portrait rounded" '
            f'src="{portrait_url}" alt="{character.character_name}" loading="lazy">'
        )

    return portrait_url


def get_type_render_url_from_type_id(
    evetype_id: int, size: int = 32, evetype_name: str = None, as_html: bool = False
) -> str:
    """
    Get type render from evetype_id

    :param evetype_id:
    :type evetype_id:
    :param size:
    :type size:
    :param evetype_name:
    :type evetype_name:
    :param as_html:
    :type as_html:
    :return:
    :rtype:
    """

    render_url = type_render_url(type_id=evetype_id, size=size)

    if as_html:
        alt_tag = f' alt="{evetype_name}"' if evetype_name else ""

        return (
            '<img class="aasrp-evetype-icon rounded" '
            f'src="{render_url}"{alt_tag} loading="lazy">'
        )

    return render_url



# Django
from django.contrib.auth.models import Permission, User
from django.template.loader import render_to_string
from django.test import TestCase
from django.urls import reverse
from django.utils import timezone
from django.utils.safestring import SafeString

# AA SRP
from aasrp.helper.icons import (
    copy_to_clipboard_icon,
    dashboard_action_icons,
    get_srp_request_action_icons,
    get_srp_request_status_icon,
)
from aasrp.models import SrpLink, SrpRequest
from aasrp.tests.utils import get_permission_content_type


class CopyToClipboardIconTests(TestCase):
    """
    Test cases for the copy_to_clipboard_icon function.
    """

    def test_copy_to_clipboard_icon_renders_correctly(self):
        """
        Test that the copy_to_clipboard_icon function renders the correct HTML.

        :return:
        :rtype:
        """

        data = "Sample data"
        title = "Copy this data"
        expected_html = render_to_string(
            template_name="aasrp/partials/common/copy-to-clipboard-icon.html",
            context={"data": data, "title": title},
        )

        result = copy_to_clipboard_icon(data, title)

        self.assertIsInstance(result, SafeString)
        self.assertEqual(result, expected_html)

    def test_copy_to_clipboard_icon_handles_empty_data(self):
        """
        Test that the copy_to_clipboard_icon function handles empty data correctly.

        :return:
        :rtype:
        """

        data = ""
        title = "Copy this data"
        expected_html = render_to_string(
            template_name="aasrp/partials/common/copy-to-clipboard-icon.html",
            context={"data": data, "title": title},
        )

        result = copy_to_clipboard_icon(data, title)

        self.assertIsInstance(result, SafeString)
        self.assertEqual(result, expected_html)

    def test_copy_to_clipboard_icon_handles_empty_title(self):
        """
        Test that the copy_to_clipboard_icon function handles empty title correctly.

        :return:
        :rtype:
        """

        data = "Sample data"
        title = ""
        expected_html = render_to_string(
            template_name="aasrp/partials/common/copy-to-clipboard-icon.html",
            context={"data": data, "title": title},
        )

        result = copy_to_clipboard_icon(data, title)

        self.assertIsInstance(result, SafeString)
        self.assertEqual(result, expected_html)

    def test_copy_to_clipboard_icon_handles_special_characters(self):
        """
        Test that the copy_to_clipboard_icon function handles special characters correctly.

        :return:
        :rtype:
        """

        data = "<script>alert('test');</script>"
        title = "Copy <script> tag"
        expected_html = render_to_string(
            template_name="aasrp/partials/common/copy-to-clipboard-icon.html",
            context={"data": data, "title": title},
        )

        result = copy_to_clipboard_icon(data, title)

        self.assertIsInstance(result, SafeString)
        self.assertEqual(result, expected_html)


class TestGetSrpRequestActionIcons(TestCase):
    """
    Test cases for the get_srp_request_action_icons function.
    """

    def test_srp_request_action_icons_for_active_srp_link_with_manage_srp(self):
        """
        Test that the get_srp_request_action_icons function returns the correct icons
        for SRP requests for an active SRP link and a user with the manage_srp permission.

        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)

        srp_link = SrpLink.objects.create(
            srp_name="Test SRP",
            srp_status=SrpLink.Status.ACTIVE,
            fleet_time=timezone.now(),  # Add a valid fleet_time
            srp_code="test_srp_code",  # Add a valid srp_code
        )
        srp_request = SrpRequest.objects.create(
            srp_link=srp_link,
            request_status=SrpRequest.Status.PENDING,
            request_code="test_srp_request_code",  # Add a valid srp_request_code
        )

        # Create a request object
        request = self.client.get(
            reverse("aasrp:view_srp_requests", kwargs={"srp_code": "test_srp_code"})
        ).wsgi_request

        result = get_srp_request_action_icons(request, srp_link, srp_request)

        view_kwargs = {
            "srp_code": "test_srp_code",
            "srp_request_code": "test_srp_request_code",
        }

        # SRP Info
        self.assertIn("fa-solid fa-circle-info", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_additional_information", kwargs=view_kwargs)}"',
            result,
        )

        # Approve
        self.assertIn("fa-solid fa-check", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_approve", kwargs=view_kwargs)}"',
            result,
        )

        # Deny
        self.assertIn("fa-solid fa-ban", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_deny", kwargs=view_kwargs)}"',
            result,
        )

        # Remove
        self.assertIn("fa-solid fa-trash-can", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_remove", kwargs=view_kwargs)}"',
            result,
        )

    def test_srp_request_action_icons_for_active_srp_link_with_manage_srp_requests(
        self,
    ):
        """
        Test that the get_srp_request_action_icons function returns the correct icons
        for SRP requests for an active SRP link and a user with the manage_srp_requests permission.

        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp_requests"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)

        srp_link = SrpLink.objects.create(
            srp_name="Test SRP",
            srp_status=SrpLink.Status.ACTIVE,
            fleet_time=timezone.now(),  # Add a valid fleet_time
            srp_code="test_srp_code",  # Add a valid srp_code
        )
        srp_request = SrpRequest.objects.create(
            srp_link=srp_link,
            request_status=SrpRequest.Status.PENDING,
            request_code="test_srp_request_code",  # Add a valid srp_request_code
        )

        # Create a request object
        request = self.client.get(
            reverse("aasrp:view_srp_requests", kwargs={"srp_code": "test_srp_code"})
        ).wsgi_request

        result = get_srp_request_action_icons(request, srp_link, srp_request)

        view_kwargs = {
            "srp_code": "test_srp_code",
            "srp_request_code": "test_srp_request_code",
        }

        # SRP Info
        self.assertIn("fa-solid fa-circle-info", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_additional_information", kwargs=view_kwargs)}"',
            result,
        )

        # Approve
        self.assertIn("fa-solid fa-check", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_approve", kwargs=view_kwargs)}"',
            result,
        )

        # Deny
        self.assertIn("fa-solid fa-ban", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_deny", kwargs=view_kwargs)}"',
            result,
        )

        # Remove
        self.assertNotIn("fa-solid fa-trash-can", result)
        self.assertNotIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_remove", kwargs=view_kwargs)}"',
            result,
        )

    def test_srp_request_action_icons_for_closed_srp_link_with_manage_srp(self):
        """
        Test that the get_srp_request_action_icons function returns the correct icons
        for SRP requests for a closed SRP link and a user with the manage_srp permission.

        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)

        srp_link = SrpLink.objects.create(
            srp_name="Test SRP",
            srp_status=SrpLink.Status.CLOSED,
            fleet_time=timezone.now(),  # Add a valid fleet_time
            srp_code="test_srp_code",  # Add a valid srp_code
        )
        srp_request = SrpRequest.objects.create(
            srp_link=srp_link,
            request_status=SrpRequest.Status.APPROVED,
            request_code="test_srp_request_code",  # Add a valid srp_request_code
        )

        # Create a request object
        request = self.client.get(
            reverse("aasrp:view_srp_requests", kwargs={"srp_code": "test_srp_code"})
        ).wsgi_request

        result = get_srp_request_action_icons(request, srp_link, srp_request)

        view_kwargs = {
            "srp_code": "test_srp_code",
            "srp_request_code": "test_srp_request_code",
        }

        # SRP Info
        self.assertIn("fa-solid fa-circle-info", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_additional_information", kwargs=view_kwargs)}"',
            result,
        )

        # Approve
        self.assertIn("fa-solid fa-check", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_approve", kwargs=view_kwargs)}"',
            result,
        )

        # Deny
        self.assertIn("fa-solid fa-ban", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_deny", kwargs=view_kwargs)}"',
            result,
        )

        # Remove
        self.assertIn("fa-solid fa-trash-can", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_remove", kwargs=view_kwargs)}"',
            result,
        )

    def test_srp_request_action_icons_for_closed_srp_link_with_manage_srp_requests(
        self,
    ):
        """
        Test that the get_srp_request_action_icons function returns the correct icons
        for SRP requests for a closed SRP link and a user with the manage_srp_requests permission.

        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp_requests"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)

        srp_link = SrpLink.objects.create(
            srp_name="Test SRP",
            srp_status=SrpLink.Status.CLOSED,
            fleet_time=timezone.now(),  # Add a valid fleet_time
            srp_code="test_srp_code",  # Add a valid srp_code
        )
        srp_request = SrpRequest.objects.create(
            srp_link=srp_link,
            request_status=SrpRequest.Status.APPROVED,
            request_code="test_srp_request_code",  # Add a valid srp_request_code
        )

        # Create a request object
        request = self.client.get(
            reverse("aasrp:view_srp_requests", kwargs={"srp_code": "test_srp_code"})
        ).wsgi_request

        result = get_srp_request_action_icons(request, srp_link, srp_request)

        view_kwargs = {
            "srp_code": "test_srp_code",
            "srp_request_code": "test_srp_request_code",
        }

        # SRP Info
        self.assertIn("fa-solid fa-circle-info", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_additional_information", kwargs=view_kwargs)}"',
            result,
        )

        # Approve
        self.assertIn("fa-solid fa-check", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_approve", kwargs=view_kwargs)}"',
            result,
        )

        # Deny
        self.assertIn("fa-solid fa-ban", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_deny", kwargs=view_kwargs)}"',
            result,
        )

        # Remove
        self.assertNotIn("fa-solid fa-trash-can", result)
        self.assertNotIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_remove", kwargs=view_kwargs)}"',
            result,
        )

    def test_srp_request_action_icons_for_completed_srp_link(self):
        """
        Test that the get_srp_request_action_icons function returns the correct icons for SRP requests for a completed SRP link.

        :return:
        :rtype:
        """

        permission_content_type = get_permission_content_type()

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=permission_content_type, codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)

        srp_link = SrpLink.objects.create(
            srp_name="Test SRP",
            srp_status=SrpLink.Status.COMPLETED,
            fleet_time=timezone.now(),  # Add a valid fleet_time
            srp_code="test_srp_code",  # Add a valid srp_code
        )
        srp_request = SrpRequest.objects.create(
            srp_link=srp_link,
            request_status=SrpRequest.Status.APPROVED,
            request_code="test_srp_request_code",  # Add a valid srp_request_code
        )

        # Create a request object
        request = self.client.get(
            reverse("aasrp:view_srp_requests", kwargs={"srp_code": "test_srp_code"})
        ).wsgi_request

        result = get_srp_request_action_icons(request, srp_link, srp_request)

        view_kwargs = {
            "srp_code": "test_srp_code",
            "srp_request_code": "test_srp_request_code",
        }

        # SRP Info
        self.assertIn("fa-solid fa-circle-info", result)
        self.assertIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_additional_information", kwargs=view_kwargs)}"',
            result,
        )

        # Approve
        self.assertNotIn("fa-solid fa-check", result)
        self.assertNotIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_approve", kwargs=view_kwargs)}"',
            result,
        )

        # Deny
        self.assertNotIn("fa-solid fa-ban", result)
        self.assertNotIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_deny", kwargs=view_kwargs)}"',
            result,
        )

        # Remove
        self.assertNotIn("fa-solid fa-trash-can", result)
        self.assertNotIn(
            f'data-link="{reverse("aasrp:ajax_srp_request_remove", kwargs=view_kwargs)}"',
            result,
        )

    def test_srp_request_action_icons_for_user_without_permission(self):
        user = User.objects.create(username="testuser")
        self.client.force_login(user)

        srp_link = SrpLink.objects.create(
            srp_name="Test SRP",
            srp_status=SrpLink.Status.ACTIVE,
            fleet_time=timezone.now(),  # Add a valid fleet_time
            srp_code="test_srp_code",  # Add a valid srp_code
        )
        srp_request = SrpRequest.objects.create(
            srp_link=srp_link,
            request_status=SrpRequest.Status.PENDING,
            request_code="test_srp_request_code",  # Add a valid srp_request_code
        )

        # Create a request object
        request = self.client.get(
            reverse("aasrp:view_srp_requests", kwargs={"srp_code": "test_srp_code"})
        ).wsgi_request

        result = get_srp_request_action_icons(request, srp_link, srp_request)

        self.assertNotIn("fa-hand-holding-dollar", result)
        self.assertNotIn("fa-eye", result)
        self.assertNotIn("fa-newspaper", result)
        self.assertNotIn("fa-ban", result)
        self.assertNotIn("fa-trash-can", result)


class TestGetSrpRequestStatusIcon(TestCase):
    """
    Test cases for the get_srp_request_status_icon function.
    """

    def test_srp_request_status_icon_for_pending_request(self):
        """
        Test that the get_srp_request_status_icon function returns the correct icon for pending requests.

        :return:
        :rtype:
        """

        user = User.objects.create(username="testuser")
        self.client.force_login(user)

        srp_request = SrpRequest.objects.create(
            srp_link=SrpLink.objects.create(
                srp_name="Test SRP",
                srp_status=SrpLink.Status.ACTIVE,
                fleet_time=timezone.now(),
                srp_code="test_srp_code",
            ),
            request_status=SrpRequest.Status.PENDING,
            request_code="test_srp_request_code",
        )

        # Create a request object
        request = self.client.get(
            reverse("aasrp:view_srp_requests", kwargs={"srp_code": "test_srp_code"})
        ).wsgi_request

        result = get_srp_request_status_icon(request, srp_request)

        self.assertIn("btn-info", result)
        self.assertIn("fa-clock", result)
        self.assertIn("SRP request pending", result)

    def test_srp_request_status_icon_for_approved_request(self):
        """
        Test that the get_srp_request_status_icon function returns the correct icon for approved requests.

        :return:
        :rtype:
        """

        user = User.objects.create(username="testuser")
        self.client.force_login(user)

        srp_request = SrpRequest.objects.create(
            srp_link=SrpLink.objects.create(
                srp_name="Test SRP",
                srp_status=SrpLink.Status.ACTIVE,
                fleet_time=timezone.now(),
                srp_code="test_srp_code",
            ),
            request_status=SrpRequest.Status.APPROVED,
            request_code="test_srp_request_code",
        )

        # Create a request object
        request = self.client.get(
            reverse("aasrp:view_srp_requests", kwargs={"srp_code": "test_srp_code"})
        ).wsgi_request

        result = get_srp_request_status_icon(request, srp_request)

        self.assertIn("btn-success", result)
        self.assertIn("fa-thumbs-up", result)
        self.assertIn("SRP request approved", result)

    def test_srp_request_status_icon_for_rejected_request(self):
        """
        Test that the get_srp_request_status_icon function returns the correct icon for rejected requests.

        :return:
        :rtype:
        """

        user = User.objects.create(username="testuser")
        self.client.force_login(user)

        srp_request = SrpRequest.objects.create(
            srp_link=SrpLink.objects.create(
                srp_name="Test SRP",
                srp_status=SrpLink.Status.ACTIVE,
                fleet_time=timezone.now(),
                srp_code="test_srp_code",
            ),
            request_status=SrpRequest.Status.REJECTED,
            request_code="test_srp_request_code",
        )

        # Create a request object
        request = self.client.get(
            reverse("aasrp:view_srp_requests", kwargs={"srp_code": "test_srp_code"})
        ).wsgi_request

        result = get_srp_request_status_icon(request, srp_request)

        self.assertIn("btn-danger", result)
        self.assertIn("fa-thumbs-down", result)
        self.assertIn("SRP request rejected", result)


class TestDashboardActionIcons(TestCase):
    """
    Test cases for the dashboard action icons.
    """

    def test_dashboard_action_icons_for_active_srp_link_with_manage_srp(self):
        """
        Test that the dashboard_action_icons function returns the correct icons for
        active SRP links and a user with the manage_srp permission.

        :return:
        :rtype:
        """

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=get_permission_content_type(), codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)

        srp_link = SrpLink.objects.create(
            srp_name="Test SRP",
            srp_status=SrpLink.Status.ACTIVE,
            fleet_time=timezone.now(),
            srp_code="test_srp_code",
        )

        # Create a request object
        request = self.client.get(reverse("aasrp:srp_links")).wsgi_request
        result = dashboard_action_icons(request, srp_link)

        self.assertIn("fa-solid fa-hand-holding-dollar", result)
        self.assertIn("fa-solid fa-eye", result)
        self.assertIn("fa-regular fa-newspaper", result)
        self.assertIn("fa-solid fa-ban", result)
        self.assertIn("fa-regular fa-trash-can", result)

    def test_dashboard_action_icons_for_active_srp_link_without_manage_permissions(
        self,
    ):
        """
        Test that the dashboard_action_icons function returns the correct icons for
        active SRP links and a user without the manage_srp permission.

        :return:
        :rtype:
        """

        user = User.objects.create(username="testuser")
        self.client.force_login(user)

        srp_link = SrpLink.objects.create(
            srp_name="Test SRP",
            srp_status=SrpLink.Status.ACTIVE,
            fleet_time=timezone.now(),
            srp_code="test_srp_code",
        )

        # Create a request object
        request = self.client.get(reverse("aasrp:srp_links")).wsgi_request
        result = dashboard_action_icons(request, srp_link)

        self.assertIn("fa-solid fa-hand-holding-dollar", result)
        self.assertNotIn("fa-solid fa-eye", result)
        self.assertNotIn("fa-regular fa-newspaper", result)
        self.assertNotIn("fa-solid fa-ban", result)
        self.assertNotIn("fa-regular fa-trash-can", result)

    def test_dashboard_action_icons_for_closed_srp_link_with_manage_srp(self):
        """
        Test that the dashboard_action_icons function returns the correct icons for
        closed SRP links and a user with the manage_srp permission.

        :return:
        :rtype:
        """

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=get_permission_content_type(), codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)

        srp_link = SrpLink.objects.create(
            srp_name="Test SRP",
            srp_status=SrpLink.Status.CLOSED,
            fleet_time=timezone.now(),
            srp_code="test_srp_code",
        )

        # Create a request object
        request = self.client.get(reverse("aasrp:srp_links")).wsgi_request
        result = dashboard_action_icons(request, srp_link)

        self.assertIn("fa-solid fa-eye", result)
        self.assertIn("fa-solid fa-check", result)
        self.assertIn("fa-regular fa-trash-can", result)

    def test_dashboard_action_icons_for_completed_srp_link_with_manage_srp(self):
        """
        Test that the dashboard_action_icons function returns the correct icons for
        completed SRP links and a user with the manage_srp permission.

        :return:
        :rtype:
        """

        user = User.objects.create(username="testuser")
        permission = Permission.objects.get(
            content_type=get_permission_content_type(), codename="manage_srp"
        )
        user.user_permissions.add(permission)
        self.client.force_login(user)

        srp_link = SrpLink.objects.create(
            srp_name="Test SRP",
            srp_status=SrpLink.Status.COMPLETED,
            fleet_time=timezone.now(),
            srp_code="test_srp_code",
        )

        # Create a request object
        request = self.client.get(reverse("aasrp:srp_links")).wsgi_request
        result = dashboard_action_icons(request, srp_link)

        self.assertIn("fa-solid fa-eye", result)
        self.assertNotIn("fa-regular fa-newspaper", result)
        self.assertNotIn("fa-solid fa-ban", result)
        self.assertNotIn("fa-regular fa-trash-can", result)



"""
SRP Manager
"""

# pylint: disable=cyclic-import

# Standard Library
from typing import Any

# Third Party
import requests

# Django
from django.contrib.auth.models import User
from django.db import models

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA SRP
from aasrp import __title__
from aasrp.constants import KILLBOARD_DATA, UserAgent
from aasrp.providers import esi

logger = LoggerAddTag(my_logger=get_extension_logger(__name__), prefix=__title__)


class SrpRequestManager(models.Manager):
    """
    SrpRequestManager
    """

    @staticmethod
    def get_kill_id(killboard_link: str) -> str:
        """
        Get killmail ID from zKillboard link

        :param killboard_link:
        :type killboard_link:
        :return:
        :rtype:
        """

        num_set = "0123456789"
        kill_id = "".join(c for c in killboard_link if c in num_set)

        return kill_id

    @staticmethod
    def get_kill_data(kill_id: str) -> tuple[int, int, int]:
        """
        Get kill data from zKillboard

        :param kill_id:
        :type kill_id:
        :return:
        :rtype:
        """

        zkillboard_api_url = KILLBOARD_DATA["zKillboard"]["api_url"]
        killmail_api_url = f"{zkillboard_api_url}killID/{kill_id}/"

        try:
            response = requests.get(
                url=killmail_api_url,
                headers={
                    "User-Agent": UserAgent.REQUESTS.value,
                    "Content-Type": "application/json",
                },
                timeout=5,
            )
            response.raise_for_status()
            result_killmails = response.json()

            result = next(
                (
                    killmail
                    for killmail in result_killmails
                    if killmail.get("killmail_id") == int(kill_id)
                ),
                None,
            )

            if not result:
                raise ValueError(
                    "No kill mail information found in zKillboard's API response."
                )

            killmail_id = result.get("killmail_id")
            killmail_hash = result.get("zkb", {}).get("hash")
            esi_killmail = esi.client.Killmails.get_killmails_killmail_id_killmail_hash(
                killmail_id=killmail_id, killmail_hash=killmail_hash
            ).result()

        except (requests.HTTPError, requests.Timeout) as exc:
            logger.warning(f"Error fetching kill mail details: {exc}", exc_info=True)

            raise ValueError(str(exc)) from exc
        except Exception as exc:
            raise ValueError("Invalid Kill ID or Hash.") from exc

        # AA SRP
        from aasrp.models import Setting  # pylint: disable=import-outside-toplevel

        loss_value_field = Setting.objects.get_setting(Setting.Field.LOSS_VALUE_SOURCE)
        ship_type = esi_killmail["victim"]["ship_type_id"]
        ship_value = result.get("zkb", {}).get(loss_value_field, 0)
        victim_id = esi_killmail["victim"]["character_id"]

        logger.debug(
            f"Kill ID {kill_id}: Ship type = {ship_type}, Loss value = {ship_value}"
        )

        return ship_type, ship_value, victim_id

    @staticmethod
    def pending_requests_count_for_user(user: User) -> int | None:
        """
        Returns the number of open SRP requests for given user or None if user has no permission

        :param user:
        :type user:
        :return:
        :rtype:
        """

        # AA SRP
        from aasrp.models import SrpRequest  # pylint: disable=import-outside-toplevel

        if user.has_perm(perm="aasrp.manage_srp") or user.has_perm(
            perm="aasrp.manage_srp_requests"
        ):
            return SrpRequest.objects.filter(
                request_status=SrpRequest.Status.PENDING
            ).count()

        return None

    @staticmethod
    def get_insurance_for_ship_type(ship_type_id: int) -> dict | None:
        """
        Getting insurance for a given ship type ID from ESI

        :param ship_type_id:
        :type ship_type_id:
        :return:
        :rtype:
        """

        insurance = next(
            (
                i
                for i in list(esi.client.Insurance.get_insurance_prices().result())
                if i["type_id"] == ship_type_id
            ),
            None,
        )

        return insurance


class SettingQuerySet(models.QuerySet):
    """
    SettingQuerySet
    """

    def delete(self):
        """
        Delete action

        Override:
            We don't allow deletion here, so we make sure the object
            is saved again and not deleted

        :return:
        :rtype:
        """

        return super().update()


class SettingManager(models.Manager):
    """
    SettingManager
    """

    def get_setting(self, setting_key: str) -> Any:
        """
        Return the value for given setting key

        :param setting_key:
        :type setting_key:
        :return:
        :rtype:
        """

        return getattr(self.first(), setting_key)

    def get_queryset(self) -> SettingQuerySet:
        """
        Get a Setting queryset

        :return:
        :rtype:
        """

        return SettingQuerySet(self.model)



# Generated by Django 4.0.10 on 2023-09-22 22:02

# Django
import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

# AA SRP
import aasrp.models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("eveuniverse", "0010_alter_eveindustryactivityduration_eve_type_and_more"),
        ("eveonline", "0017_alliance_and_corp_names_are_not_unique"),
        ("aasrp", "0011_default_settings"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="fleettype",
            options={
                "default_permissions": (),
                "verbose_name": "Fleet type",
                "verbose_name_plural": "Fleet types",
            },
        ),
        migrations.AlterModelOptions(
            name="insurance",
            options={
                "default_permissions": (),
                "verbose_name": "Ship insurance",
                "verbose_name_plural": "Ship insurances",
            },
        ),
        migrations.AlterModelOptions(
            name="setting",
            options={
                "default_permissions": (),
                "verbose_name": "Setting",
                "verbose_name_plural": "Settings",
            },
        ),
        migrations.AlterModelOptions(
            name="srplink",
            options={
                "default_permissions": (),
                "verbose_name": "SRP link",
                "verbose_name_plural": "SRP links",
            },
        ),
        migrations.AlterModelOptions(
            name="usersetting",
            options={
                "default_permissions": (),
                "verbose_name": "User settings",
                "verbose_name_plural": "User settings",
            },
        ),
        migrations.AlterField(
            model_name="insurance",
            name="insurance_cost",
            field=models.FloatField(verbose_name="Insurance cost"),
        ),
        migrations.AlterField(
            model_name="insurance",
            name="insurance_level",
            field=models.CharField(
                default="", max_length=254, verbose_name="Insurance level"
            ),
        ),
        migrations.AlterField(
            model_name="insurance",
            name="insurance_payout",
            field=models.FloatField(verbose_name="Insurance payout"),
        ),
        migrations.AlterField(
            model_name="insurance",
            name="srp_request",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="insurance",
                to="aasrp.srprequest",
                verbose_name="SRP request",
            ),
        ),
        migrations.AlterField(
            model_name="requestcomment",
            name="comment_time",
            field=models.DateTimeField(
                blank=True,
                default=django.utils.timezone.now,
                null=True,
                verbose_name="Comment time",
            ),
        ),
        migrations.AlterField(
            model_name="requestcomment",
            name="comment_type",
            field=models.CharField(
                choices=[
                    ("Comment", "Comment"),
                    ("Request Added", "SRP request added"),
                    ("Request Information", "Additional information"),
                    ("Reject Reason", "Reject reason"),
                    ("Status Changed", "Status changed"),
                    ("Reviser Comment", "Reviser comment"),
                ],
                default="Comment",
                max_length=19,
                verbose_name="Comment type",
            ),
        ),
        migrations.AlterField(
            model_name="requestcomment",
            name="new_status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("Pending", "Pending"),
                    ("Approved", "Approved"),
                    ("Rejected", "Rejected"),
                ],
                default="",
                max_length=8,
                verbose_name="New SRP request status",
            ),
        ),
        migrations.AlterField(
            model_name="requestcomment",
            name="srp_request",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="srp_request_comments",
                to="aasrp.srprequest",
                verbose_name="SRP request",
            ),
        ),
        migrations.AlterField(
            model_name="setting",
            name="srp_team_discord_channel_id",
            field=models.PositiveBigIntegerField(
                blank=True,
                default=None,
                null=True,
                verbose_name="SRP team Discord channel ID",
            ),
        ),
        migrations.AlterField(
            model_name="srplink",
            name="aar_link",
            field=models.CharField(
                blank=True, default="", max_length=254, verbose_name="AAR link"
            ),
        ),
        migrations.AlterField(
            model_name="srplink",
            name="fleet_commander",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="eveonline.evecharacter",
                verbose_name="Fleet commander",
            ),
        ),
        migrations.AlterField(
            model_name="srplink",
            name="fleet_time",
            field=models.DateTimeField(verbose_name="Fleet time"),
        ),
        migrations.AlterField(
            model_name="srplink",
            name="fleet_type",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="The SRP link fleet type, if it's set",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="aasrp.fleettype",
                verbose_name="Fleet type",
            ),
        ),
        migrations.AlterField(
            model_name="srplink",
            name="srp_code",
            field=models.CharField(default="", max_length=16, verbose_name="SRP code"),
        ),
        migrations.AlterField(
            model_name="srplink",
            name="srp_name",
            field=models.CharField(default="", max_length=254, verbose_name="SRP name"),
        ),
        migrations.AlterField(
            model_name="srplink",
            name="srp_status",
            field=models.CharField(
                choices=[
                    ("Active", "Active"),
                    ("Closed", "Closed"),
                    ("Completed", "Completed"),
                ],
                default="Active",
                max_length=9,
                verbose_name="SRP status",
            ),
        ),
        migrations.AlterField(
            model_name="srprequest",
            name="additional_info",
            field=models.TextField(
                blank=True, default="", verbose_name="Additional information"
            ),
        ),
        migrations.AlterField(
            model_name="srprequest",
            name="creator",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Who created the SRP request?",
                null=True,
                on_delete=models.SET(aasrp.models.get_sentinel_user),
                related_name="+",
                to=settings.AUTH_USER_MODEL,
                verbose_name="Creator",
            ),
        ),
        migrations.AlterField(
            model_name="srprequest",
            name="killboard_link",
            field=models.CharField(
                default="", max_length=254, verbose_name="Killboard link"
            ),
        ),
        migrations.AlterField(
            model_name="srprequest",
            name="loss_amount",
            field=models.BigIntegerField(default=0, verbose_name="Loss amount"),
        ),
        migrations.AlterField(
            model_name="srprequest",
            name="payout_amount",
            field=models.BigIntegerField(default=0, verbose_name="Payout amount"),
        ),
        migrations.AlterField(
            model_name="srprequest",
            name="post_time",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Request time"
            ),
        ),
        migrations.AlterField(
            model_name="srprequest",
            name="reject_info",
            field=models.TextField(
                blank=True, default="", verbose_name="Reject reason"
            ),
        ),
        migrations.AlterField(
            model_name="srprequest",
            name="request_code",
            field=models.CharField(
                default="", max_length=254, verbose_name="Request code"
            ),
        ),
        migrations.AlterField(
            model_name="srprequest",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("Pending", "Pending"),
                    ("Approved", "Approved"),
                    ("Rejected", "Rejected"),
                ],
                default="Pending",
                max_length=8,
                verbose_name="Request status",
            ),
        ),
        migrations.AlterField(
            model_name="srprequest",
            name="ship",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="srp_requests",
                to="eveuniverse.evetype",
                verbose_name="Ship type",
            ),
        ),
        migrations.AlterField(
            model_name="srprequest",
            name="ship_name",
            field=models.CharField(
                default="", max_length=254, verbose_name="Ship type"
            ),
        ),
        migrations.AlterField(
            model_name="srprequest",
            name="srp_link",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="srp_requests",
                to="aasrp.srplink",
                verbose_name="SRP link",
            ),
        ),
        migrations.AlterField(
            model_name="usersetting",
            name="disable_notifications",
            field=models.BooleanField(
                default=False, verbose_name="Disable notifications"
            ),
        ),
    ]



# Generated by Django 3.1.7 on 2021-03-13 23:36

# Django
from django.conf import settings
from django.db import migrations, models

# AA SRP
import aasrp.models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("aasrp", "0003_aasrprequest_reject_info"),
    ]

    operations = [
        migrations.CreateModel(
            name="AaSrpUserSettings",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("disable_notifications", models.BooleanField(default=False)),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        null=True,
                        on_delete=models.SET(aasrp.models.get_sentinel_user),
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "User Settings",
                "verbose_name_plural": "User Settings",
                "default_permissions": (),
            },
        ),
    ]



"""
Ajax views
"""

# Standard Library
import json

# Django
from django.contrib.auth.decorators import permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.db.models import Q
from django.http import HttpResponse, JsonResponse
from django.shortcuts import render
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.authentication.decorators import permissions_required
from allianceauth.framework.api.user import get_main_character_name_from_user
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag
from app_utils.urls import reverse_absolute

# AA SRP
from aasrp import __title__
from aasrp.form import (
    SrpRequestAcceptForm,
    SrpRequestAcceptRejectedForm,
    SrpRequestPayoutForm,
    SrpRequestRejectForm,
)
from aasrp.helper.character import get_formatted_character_name
from aasrp.helper.eve_images import get_type_render_url_from_type_id
from aasrp.helper.icons import (
    copy_to_clipboard_icon,
    dashboard_action_icons,
    get_srp_request_action_icons,
    get_srp_request_details_icon,
    get_srp_request_status_icon,
)
from aasrp.helper.notification import notify_requester
from aasrp.helper.srp_data import (
    attempt_to_re_add_ship_information_to_request,
    localized_isk_value,
    payout_amount_html,
    request_code_html,
    request_fleet_details_html,
)
from aasrp.helper.user import get_user_settings
from aasrp.models import RequestComment, SrpLink, SrpRequest

logger = LoggerAddTag(my_logger=get_extension_logger(__name__), prefix=__title__)


@permission_required("aasrp.basic_access")
def dashboard_srp_links_data(
    request: WSGIRequest, show_all_links: bool = False
) -> JsonResponse:
    """
    Ajax request :: Get all active SRP links

    :param request:
    :type request:
    :param show_all_links:
    :type show_all_links:
    :return:
    :rtype:
    """

    data = []

    srp_links = SrpLink.objects.prefetch_related(
        "fleet_commander",
        "creator",
        "creator__profile__main_character",
        "srp_requests",
    ).all()

    if not show_all_links:
        srp_links = srp_links.filter(srp_status=SrpLink.Status.ACTIVE)

    for srp_link in srp_links:
        l10n_link = _("Link")
        aar_link = (
            f'<a href="{srp_link.aar_link}" target="_blank">{l10n_link}</a>'
            if srp_link.aar_link
            else ""
        )

        srp_code_html = srp_link.srp_code

        if srp_link.srp_status == SrpLink.Status.ACTIVE:
            srp_link_href = reverse_absolute(
                viewname="aasrp:request_srp", args=[srp_link.srp_code]
            )
            title = _("Copy SRP link to clipboard")
            copy_icon = copy_to_clipboard_icon(data=srp_link_href, title=title)
            srp_code_html += f"<sup>{copy_icon}</sup>"

        fleet_type = srp_link.fleet_type.name if srp_link.fleet_type else ""

        data.append(
            {
                "srp_name": srp_link.srp_name,
                "creator": get_main_character_name_from_user(user=srp_link.creator),
                "fleet_time": srp_link.fleet_time,
                "fleet_type": fleet_type,
                "fleet_doctrine": srp_link.fleet_doctrine,
                "aar_link": aar_link,
                "srp_code": {"display": srp_code_html, "sort": srp_link.srp_code},
                "srp_costs": srp_link.total_cost,
                "srp_costs_html": {
                    "display": localized_isk_value(srp_link.total_cost),
                    "sort": srp_link.total_cost,
                },
                "srp_status": srp_link.srp_status,
                "pending_requests": srp_link.pending_requests,
                "actions": dashboard_action_icons(request=request, srp_link=srp_link),
            }
        )

    return JsonResponse(data=data, safe=False)


@permission_required("aasrp.basic_access")
def dashboard_user_srp_requests_data(request: WSGIRequest) -> JsonResponse:
    """
    Ajax request :: Get user srp requests

    :param request:
    :type request:
    :return:
    :rtype:
    """

    data = []

    requests = (
        SrpRequest.objects.filter(creator=request.user)
        # .filter(ship__isnull=False)
        .prefetch_related(
            "creator",
            "creator__profile__main_character",
            "character",
            "srp_link",
            "srp_link__creator",
            "srp_link__creator__profile__main_character",
            "ship",
        )
    )

    for srp_request in requests:
        killboard_link = ""

        if srp_request.killboard_link:
            try:
                ship_render_icon_html = get_type_render_url_from_type_id(
                    evetype_id=srp_request.ship_id,
                    evetype_name=srp_request.ship.name,
                    size=32,
                    as_html=True,
                )
            except AttributeError:
                # For some reason, it seems the ship has been removed from EveType
                # table, attempt to add it again …
                srp_request = attempt_to_re_add_ship_information_to_request(
                    srp_request=srp_request
                )

                ship_render_icon_html = get_type_render_url_from_type_id(
                    evetype_id=srp_request.ship_id,
                    evetype_name=srp_request.ship.name,
                    size=32,
                    as_html=True,
                )

            zkb_link = srp_request.killboard_link
            zkb_link_text = srp_request.ship.name
            killboard_link = (
                f'<a href="{zkb_link}" target="_blank">'
                f"{ship_render_icon_html}"
                f"<span>{zkb_link_text}</span>"
                "</a>"
            )

        srp_request_status_icon = get_srp_request_status_icon(
            request=request, srp_request=srp_request
        )

        srp_request_details_icon = get_srp_request_details_icon(
            request=request, srp_link=srp_request.srp_link, srp_request=srp_request
        )

        character_display = get_formatted_character_name(
            character=srp_request.character, with_portrait=True
        )
        character_sort = get_formatted_character_name(character=srp_request.character)

        data.append(
            {
                "request_time": srp_request.post_time,
                "character": srp_request.character.character_name,
                "character_html": {
                    "display": character_display,
                    "sort": character_sort,
                },
                "fleet_name_html": {
                    "display": request_fleet_details_html(srp_request=srp_request),
                    "sort": srp_request.srp_link.srp_name,
                },
                "srp_code": srp_request.srp_link.srp_code,
                "request_code": srp_request.request_code,
                "ship": srp_request.ship.name,
                "ship_html": {
                    "display": killboard_link,
                    "sort": srp_request.ship.name,
                },
                "zkb_link": killboard_link,
                "zkb_loss_amount_html": {
                    "display": localized_isk_value(srp_request.loss_amount),
                    "sort": srp_request.loss_amount,
                },
                "payout_amount": srp_request.payout_amount,
                "payout_amount_html": {
                    "display": localized_isk_value(srp_request.payout_amount),
                    "sort": srp_request.loss_amount,
                },
                "request_status_icon": (
                    srp_request_details_icon + srp_request_status_icon
                ),
                "request_status": srp_request.get_request_status_display(),
            }
        )

    return JsonResponse(data=data, safe=False)


@permissions_required(("aasrp.manage_srp", "aasrp.manage_srp_requests"))
def srp_link_view_requests_data(request: WSGIRequest, srp_code: str) -> JsonResponse:
    """
    Ajax request :: Get datatable data

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :return:
    :rtype:
    """

    data = []

    srp_requests = SrpRequest.objects.filter(
        srp_link__srp_code__iexact=srp_code
    ).prefetch_related(
        "srp_link",
        "srp_link__creator",
        "srp_link__creator__profile__main_character",
        "character",
        "ship",
    )

    for srp_request in srp_requests:
        killboard_link = ""

        if srp_request.killboard_link:
            try:
                ship_render_icon_html = get_type_render_url_from_type_id(
                    evetype_id=srp_request.ship_id,
                    evetype_name=srp_request.ship.name,
                    size=32,
                    as_html=True,
                )
            except AttributeError:
                # For some reason, it seems the ship has been removed from EveType
                # table, attempt to add it again …
                srp_request = attempt_to_re_add_ship_information_to_request(srp_request)

                ship_render_icon_html = get_type_render_url_from_type_id(
                    evetype_id=srp_request.ship_id,
                    evetype_name=srp_request.ship.name,
                    size=32,
                    as_html=True,
                )

            killboard_link = (
                f'<a href="{srp_request.killboard_link}" target="_blank">'
                f"{ship_render_icon_html}"
                f"<span>{srp_request.ship.name}</span></a>"
            )

        data.append(
            {
                "request_time": srp_request.post_time,
                "requester": get_main_character_name_from_user(srp_request.creator),
                "character_html": {
                    "display": get_formatted_character_name(
                        character=srp_request.character,
                        with_portrait=True,
                        with_copy_icon=True,
                    ),
                    "sort": srp_request.character.character_name,
                },
                "character": srp_request.character.character_name,
                "request_code_html": {
                    "display": request_code_html(request_code=srp_request.request_code),
                    "sort": srp_request.request_code,
                },
                "request_code": srp_request.request_code,
                "srp_code": srp_request.srp_link.srp_code,
                "ship_html": {"display": killboard_link, "sort": srp_request.ship.name},
                "ship": srp_request.ship.name,
                "zkb_link": killboard_link,
                "zkb_loss_amount_html": {
                    "display": localized_isk_value(srp_request.loss_amount),
                    "sort": srp_request.loss_amount,
                },
                "zbk_loss_amount": srp_request.loss_amount,
                "payout_amount_html": {
                    "display": payout_amount_html(
                        payout_amount=srp_request.payout_amount
                    ),
                    "sort": srp_request.payout_amount,
                },
                "payout_amount": srp_request.payout_amount,
                "request_status_icon": get_srp_request_status_icon(
                    request=request, srp_request=srp_request
                ),
                "actions": get_srp_request_action_icons(
                    request=request,
                    srp_link=srp_request.srp_link,
                    srp_request=srp_request,
                ),
                "request_status_translated": srp_request.get_request_status_display(),
                "request_status": srp_request.request_status,
            }
        )

    return JsonResponse(data=data, safe=False)


@permission_required("aasrp.basic_access")
def srp_request_additional_information(
    request: WSGIRequest, srp_code: str, srp_request_code: str
) -> HttpResponse:
    """
    Ajax Call :: Get additional information for an SRP request

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :param srp_request_code:
    :type srp_request_code:
    :return:
    :rtype:
    """

    srp_request = SrpRequest.objects.get(
        srp_link__srp_code=srp_code, request_code=srp_request_code
    )

    insurance_information = srp_request.insurance.filter(srp_request=srp_request)

    character = get_formatted_character_name(
        character=srp_request.character, with_portrait=True, portrait_size=64
    )

    try:
        ship_render_icon_html = get_type_render_url_from_type_id(
            evetype_id=srp_request.ship_id,
            evetype_name=srp_request.ship.name,
            size=64,
            as_html=True,
        )
    except AttributeError:
        # For some reason, it seems the ship has been removed from EveType
        # table, attempt to add it again …
        srp_request = attempt_to_re_add_ship_information_to_request(
            srp_request=srp_request
        )

        ship_render_icon_html = get_type_render_url_from_type_id(
            evetype_id=srp_request.ship_id,
            evetype_name=srp_request.ship.name,
            size=64,
            as_html=True,
        )

    request_status_banner_alert_level = {
        SrpRequest.Status.APPROVED: "success",
        SrpRequest.Status.REJECTED: "danger",
    }.get(srp_request.request_status, "info")

    additional_info = RequestComment.objects.filter(
        srp_request=srp_request, comment_type=RequestComment.Type.REQUEST_INFO
    ).first()
    additional_info = additional_info.comment if additional_info else ""

    request_history = RequestComment.objects.filter(
        ~Q(comment_type=RequestComment.Type.REQUEST_INFO),
        srp_request=srp_request,
    ).order_by("pk")

    data = {
        "srp_request": srp_request,
        "ship_render_icon_html": ship_render_icon_html,
        "ship_type": srp_request.ship.name,
        "requester": get_main_character_name_from_user(user=srp_request.creator),
        "character": character,
        "additional_info": additional_info,
        "request_status_banner_alert_level": request_status_banner_alert_level,
        "request_status": srp_request.get_request_status_display(),
        "insurance_information": insurance_information,
        "request_history": request_history,
    }

    return render(
        request=request,
        template_name="aasrp/ajax-render/srp-request-additional-information.html",
        context=data,
    )


@permissions_required(("aasrp.manage_srp", "aasrp.manage_srp_requests"))
def srp_request_change_payout(
    request: WSGIRequest, srp_code: str, srp_request_code: str
) -> JsonResponse:
    """
    Ajax call :: Change SRP payout

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :param srp_request_code:
    :type srp_request_code:
    :return:
    :rtype:
    """

    if request.method == "POST":
        try:
            srp_request = SrpRequest.objects.get(
                request_code=srp_request_code, srp_link__srp_code=srp_code
            )
            form = SrpRequestPayoutForm(data=request.POST)

            if form.is_valid():
                srp_request.payout_amount = form.cleaned_data["value"]
                srp_request.save()

                return JsonResponse(data={"success": True}, safe=False)
        except SrpRequest.DoesNotExist:
            pass

    return JsonResponse(data={"success": False}, safe=False)


@permissions_required(("aasrp.manage_srp", "aasrp.manage_srp_requests"))
def srp_request_approve(  # pylint: disable=too-many-locals
    request: WSGIRequest, srp_code: str, srp_request_code: str
) -> JsonResponse:
    """
    Ajax call :: Approve SRP request

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :param srp_request_code:
    :type srp_request_code:
    :return:
    :rtype:
    """

    try:
        srp_request = SrpRequest.objects.get(
            request_code=srp_request_code, srp_link__srp_code=srp_code
        )
    except SrpRequest.DoesNotExist:
        return JsonResponse(
            data={"success": False, "message": _("No matching SRP request found")},
            safe=False,
        )

    if request.method == "POST":
        form = (
            SrpRequestAcceptForm(data=json.loads(request.body))
            if srp_request.request_status == SrpRequest.Status.PENDING
            else (
                SrpRequestAcceptRejectedForm(data=json.loads(request.body))
                if srp_request.request_status == SrpRequest.Status.REJECTED
                else None
            )
        )

        if not form.is_valid():
            return JsonResponse(
                data={"success": False, "message": _("Invalid form data")}, safe=False
            )

        requester = srp_request.creator
        srp_request.payout_amount = srp_request.payout_amount or srp_request.loss_amount

        # Create comments in bulk
        comments = [
            RequestComment(
                srp_request=srp_request,
                comment_type=RequestComment.Type.STATUS_CHANGE,
                new_status=SrpRequest.Status.APPROVED,
                creator=request.user,
            )
        ]

        reviser_comment = form.cleaned_data["comment"]

        if reviser_comment:
            comments.append(
                RequestComment(
                    srp_request=srp_request,
                    comment=reviser_comment,
                    comment_type=RequestComment.Type.REVISER_COMMENT,
                    creator=request.user,
                )
            )
        RequestComment.objects.bulk_create(comments)

        srp_request.request_status = SrpRequest.Status.APPROVED
        srp_request.save()

        # Send notification if enabled
        if not get_user_settings(user=requester).disable_notifications:
            logger.info(msg="Sending approval message to user")

            notify_requester(
                requester=requester,
                reviser=get_main_character_name_from_user(user=request.user),
                srp_request=srp_request,
                comment=reviser_comment,
            )

        return JsonResponse(
            data={"success": True, "message": _("SRP request has been approved")},
            safe=False,
        )

    return JsonResponse(
        data={"success": False, "message": _("Invalid request method")},
        safe=False,
    )


@permissions_required(("aasrp.manage_srp", "aasrp.manage_srp_requests"))
def srp_requests_bulk_approve(request: WSGIRequest, srp_code: str) -> JsonResponse:
    """
    Ajax call :: Approve multiple SRP requests

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :return:
    :rtype:
    """

    if request.method == "POST":
        request_body = json.loads(request.body)
        srp_request_codes = request_body.get("srp_request_codes")

        logger.debug(
            "Bulk approving SRP requests for code: %s, with request codes: %s",
            srp_code,
            srp_request_codes,
        )

        if not srp_request_codes or not srp_code:
            return JsonResponse(
                data={"success": False, "message": _("Invalid form data")}, safe=False
            )

        srp_requests = SrpRequest.objects.filter(
            Q(request_code__in=srp_request_codes)
            & Q(srp_link__srp_code=srp_code)
            & Q(request_status=SrpRequest.Status.PENDING)
        )

        logger.debug(
            "Found %d SRP requests to approve for code: %s",
            srp_requests.count(),
            srp_code,
        )

        if not srp_requests.exists():
            return JsonResponse(
                data={"success": False, "message": _("No matching SRP requests found")},
                safe=False,
            )

        # Prepare bulk updates
        srp_request_list = list(srp_requests)
        for srp_request in srp_request_list:
            srp_request.payout_amount = (
                srp_request.payout_amount or srp_request.loss_amount
            )
            srp_request.request_status = SrpRequest.Status.APPROVED

        # Bulk update all requests
        SrpRequest.objects.bulk_update(
            srp_request_list, ["payout_amount", "request_status"]
        )

        # Bulk create comments
        comments = [
            RequestComment(
                srp_request=srp_request,
                comment_type=RequestComment.Type.STATUS_CHANGE,
                new_status=SrpRequest.Status.APPROVED,
                creator=request.user,
            )
            for srp_request in srp_request_list
        ]
        RequestComment.objects.bulk_create(comments)

        # Send notifications
        reviser_name = get_main_character_name_from_user(user=request.user)
        for srp_request in srp_request_list:
            requester = srp_request.creator
            if not get_user_settings(user=requester).disable_notifications:
                logger.info(msg="Sending approval message to user")
                notify_requester(
                    requester=requester,
                    reviser=reviser_name,
                    srp_request=srp_request,
                    comment="",
                )

        return JsonResponse(
            data={"success": True, "message": _("SRP requests have been approved")},
            safe=False,
        )

    return JsonResponse(
        data={"success": False, "message": _("Invalid request method")},
        safe=False,
    )


@permissions_required(("aasrp.manage_srp", "aasrp.manage_srp_requests"))
def srp_request_deny(
    request: WSGIRequest, srp_code: str, srp_request_code: str
) -> JsonResponse:
    """
    Ajax call :: Deny SRP request

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :param srp_request_code:
    :type srp_request_code:
    :return:
    :rtype:
    """

    try:
        srp_request = SrpRequest.objects.get(
            request_code=srp_request_code, srp_link__srp_code=srp_code
        )
    except SrpRequest.DoesNotExist:
        return JsonResponse(
            data={"success": False, "message": _("No matching SRP request found")},
            safe=False,
        )

    if request.method == "POST":
        form = SrpRequestRejectForm(data=json.loads(request.body))

        if not form.is_valid():
            return JsonResponse(
                data={"success": False, "message": _("Invalid form data")}, safe=False
            )

        reject_info = form.cleaned_data["comment"]
        requester = srp_request.creator

        srp_request.payout_amount = 0
        srp_request.request_status = SrpRequest.Status.REJECTED
        srp_request.save()

        RequestComment.objects.bulk_create(
            [
                RequestComment(
                    srp_request=srp_request,
                    comment_type=RequestComment.Type.STATUS_CHANGE,
                    new_status=SrpRequest.Status.REJECTED,
                    creator=request.user,
                ),
                RequestComment(
                    comment=reject_info,
                    srp_request=srp_request,
                    comment_type=RequestComment.Type.REJECT_REASON,
                    creator=request.user,
                ),
            ]
        )

        if not get_user_settings(user=requester).disable_notifications:
            logger.info("Sending reject message to user")

            notify_requester(
                requester=requester,
                reviser=get_main_character_name_from_user(user=request.user),
                srp_request=srp_request,
                comment=reject_info,
                message_level="danger",
            )

        return JsonResponse(
            data={"success": True, "message": _("SRP request has been rejected")},
            safe=False,
        )

    return JsonResponse(
        data={"success": False, "message": _("Invalid request method")},
        safe=False,
    )


@permissions_required(("aasrp.manage_srp", "aasrp.manage_srp_requests"))
def srp_request_remove(
    request: WSGIRequest,  # pylint: disable=unused-argument
    srp_code: str,
    srp_request_code: str,
) -> JsonResponse:
    """
    Ajax call :: Remove SRP request

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :param srp_request_code:
    :type srp_request_code:
    :return:
    :rtype:
    """

    try:
        SrpRequest.objects.get(
            request_code=srp_request_code, srp_link__srp_code=srp_code
        ).delete()

        data = {"success": True, "message": _("SRP request has been removed")}
    except SrpRequest.DoesNotExist:
        data = {"success": False, "message": _("No matching SRP request found")}

    return JsonResponse(data=data, safe=False)


@permissions_required(("aasrp.manage_srp", "aasrp.manage_srp_requests"))
def srp_requests_bulk_remove(request: WSGIRequest, srp_code: str) -> JsonResponse:
    """
    Ajax call :: Remove multiple SRP requests

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :return:
    :rtype:
    """

    if request.method == "POST":
        request_body = json.loads(request.body)
        srp_request_codes = request_body.get("srp_request_codes")

        logger.debug(
            "Bulk removing SRP requests for code: %s, with request codes: %s",
            srp_code,
            srp_request_codes,
        )

        if not srp_request_codes or not srp_code:
            return JsonResponse(
                data={"success": False, "message": _("Invalid form data")}, safe=False
            )

        srp_requests = SrpRequest.objects.filter(
            Q(request_code__in=srp_request_codes) & Q(srp_link__srp_code=srp_code)
        )

        logger.debug(
            "Found %d SRP requests to remove for code: %s",
            srp_requests.count(),
            srp_code,
        )

        if not srp_requests.exists():
            return JsonResponse(
                data={"success": False, "message": _("No matching SRP requests found")},
                safe=False,
            )

        srp_requests.delete()

        return JsonResponse(
            data={"success": True, "message": _("SRP requests have been removed")},
            safe=False,
        )

    return JsonResponse(
        data={"success": False, "message": _("Invalid request method")},
        safe=False,
    )



"""
Test the admin interface.
"""

# Standard Library
from http import HTTPStatus
from unittest.mock import MagicMock, patch

# Django
from django.contrib import admin
from django.contrib.auth.models import User
from django.test import Client, TestCase
from django.urls import reverse

# AA SRP
from aasrp.admin import RequestCommentAdmin, SrpLinkAdmin, SrpRequestAdmin
from aasrp.models import FleetType, RequestComment


class TestFleetTypeAdmin(TestCase):
    """
    Test the FleetType admin interface.
    """

    @classmethod
    def setUpTestData(cls):
        """
        Set up test data for the admin interface tests.

        :return:
        :rtype:
        """

        cls.admin_user = User.objects.create_superuser(
            username="admin", email="admin@example.com", password="password"
        )
        cls.client = Client()
        cls.client.login(username="admin", password="password")
        cls.fleet_type = FleetType.objects.create(name="Test Fleet", is_enabled=True)

    def test_admin_page_loads(self):
        """
        Test that the admin page loads correctly.

        :return:
        :rtype:
        """

        url = reverse("admin:aasrp_fleettype_changelist")

        self.client.login(username="admin", password="password")

        response = self.client.get(url)

        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)

    def test_admin_form_saves_valid_data(self):
        """
        Test that the admin form saves valid data correctly.

        :return:
        :rtype:
        """

        url = reverse("admin:aasrp_fleettype_change", args=[self.fleet_type.pk])

        self.client.login(username="admin", password="password")

        data = {
            "name": "Updated Fleet",
            "is_enabled": False,
            "_save": "Save",  # Include the save button name to simulate form submission
        }
        response = self.client.post(path=url, data=data)

        self.assertEqual(first=response.status_code, second=HTTPStatus.FOUND)
        self.fleet_type.refresh_from_db()
        self.assertEqual(first=self.fleet_type.name, second="Updated Fleet")
        self.assertFalse(self.fleet_type.is_enabled)

    def test_activate_action_activates_selected_fleet_types(self):
        """
        Test that the activate action activates selected fleet types.

        :return:
        :rtype:
        """

        url = reverse("admin:aasrp_fleettype_changelist")

        self.client.login(username="admin", password="password")

        data = {
            "action": "activate",
            "_selected_action": [self.fleet_type.pk],
        }
        response = self.client.post(path=url, data=data, follow=True)

        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)
        self.fleet_type.refresh_from_db()
        self.assertTrue(self.fleet_type.is_enabled)

    def test_deactivate_action_deactivates_selected_fleet_types(self):
        """
        Test that the deactivate action deactivates selected fleet types.

        :return:
        :rtype:
        """

        url = reverse("admin:aasrp_fleettype_changelist")

        self.client.login(username="admin", password="password")

        data = {
            "action": "deactivate",
            "_selected_action": [self.fleet_type.pk],
        }
        response = self.client.post(path=url, data=data, follow=True)

        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)
        self.fleet_type.refresh_from_db()
        self.assertFalse(self.fleet_type.is_enabled)


class TestRequestCommentAdmin(TestCase):
    """
    Test the RequestComment admin interface.
    """

    def test_displays_correct_srp_code_in_admin(self):
        """
        Test that the SRP code is displayed correctly in the admin interface.

        :return:
        :rtype:
        """

        request_comment = MagicMock()
        request_comment.srp_request.srp_link.srp_code = "SRP123"

        admin_instance = RequestCommentAdmin(
            model=RequestComment, admin_site=admin.site
        )
        result = admin_instance._srp_code(request_comment)

        self.assertEqual(first=result, second="SRP123")

    def test_displays_correct_request_code_in_admin(self):
        """
        Test that the request code is displayed correctly in the admin interface.

        :return:
        :rtype:
        """

        request_comment = MagicMock()
        request_comment.srp_request.request_code = "REQ456"

        admin_instance = RequestCommentAdmin(
            model=RequestComment, admin_site=admin.site
        )
        result = admin_instance._request_code(request_comment)

        self.assertEqual(first=result, second="REQ456")

    def test_displays_correct_requestor_name_in_admin(self):
        """
        Test that the requestor name is displayed correctly in the admin interface.

        :return:
        :rtype:
        """

        request_comment = MagicMock()
        request_comment.srp_request.creator = MagicMock()

        with patch(
            target="aasrp.admin.get_main_character_name_from_user",
            return_value="John Doe",
        ):
            admin_instance = RequestCommentAdmin(
                model=RequestComment, admin_site=admin.site
            )
            result = admin_instance._requestor(request_comment)

        self.assertEqual(first=result, second="John Doe")

    def test_displays_correct_character_name_in_admin(self):
        request_comment = MagicMock()
        request_comment.srp_request.character.character_name = "Jane Doe"

        admin_instance = RequestCommentAdmin(
            model=RequestComment, admin_site=admin.site
        )
        result = admin_instance._character(request_comment)

        self.assertEqual(first=result, second="Jane Doe")


class TestSrpRequestAdmin(TestCase):
    """
    Test the SrpRequest admin interface.
    """

    def test_displays_correct_requestor_name_for_valid_creator(self):
        """
        Test that the requestor name is displayed correctly when the creator is valid.

        :return:
        :rtype:
        """

        srp_request = MagicMock()
        srp_request.creator = MagicMock()

        with patch(
            target="aasrp.admin.get_main_character_name_from_user",
            return_value="John Doe",
        ):
            result = SrpRequestAdmin._requestor(srp_request)

        self.assertEqual(first=result, second="John Doe")

    def test_handles_missing_creator_gracefully(self):
        """
        Test that the requestor name is None when the creator is missing.

        :return:
        :rtype:
        """

        srp_request = MagicMock()
        srp_request.creator = None

        with patch(
            target="aasrp.admin.get_main_character_name_from_user", return_value=None
        ):
            result = SrpRequestAdmin._requestor(srp_request)

        self.assertIsNone(result)

    def test_displays_correct_srp_code_for_valid_link(self):
        """
        Test that the SRP code is displayed correctly when the SRP link is valid.

        :return:
        :rtype:
        """

        srp_request = MagicMock()
        srp_request.srp_link.srp_code = "SRP123"

        admin_instance = SrpRequestAdmin(model=RequestComment, admin_site=admin.site)
        result = admin_instance._srp_code(srp_request)

        self.assertEqual(first=result, second="SRP123")


class TestSrpLinkAdmin(TestCase):
    """
    Test the SrpLink admin interface.
    """

    def test_displays_correct_creator_name_for_valid_creator(self):
        """
        Test that the creator name is displayed correctly when the creator is valid.

        :return:
        :rtype:
        """

        srp_link = MagicMock()
        srp_link.creator = MagicMock()

        with patch(
            target="aasrp.admin.get_main_character_name_from_user",
            return_value="Jane Doe",
        ):
            result = SrpLinkAdmin._creator(srp_link)

        self.assertEqual(first=result, second="Jane Doe")

    def test_handles_missing_creator_gracefully(self):
        srp_link = MagicMock()
        srp_link.creator = None

        with patch(
            target="aasrp.admin.get_main_character_name_from_user", return_value=None
        ):
            result = SrpLinkAdmin._creator(srp_link)

        self.assertIsNone(result)



"""
AA-SRP Views URLs
"""

# Django
from django.urls import path

# AA SRP
from aasrp.views import general

urls = [
    path(route="", view=general.srp_links, name="srp_links"),
    path(
        route="all/",
        view=general.srp_links,
        kwargs={"show_all_links": True},
        name="srp_links_all",
    ),
    path(
        route="my-srp-requests/",
        view=general.view_own_requests,
        name="own_srp_requests",
    ),
    path(route="my-settings/", view=general.user_settings, name="user_settings"),
    path(route="add/", view=general.srp_link_add, name="add_srp_link"),
    path(
        route="srp-link/<str:srp_code>/edit/",
        view=general.srp_link_edit,
        name="edit_srp_link",
    ),
    path(
        route="srp-link/<str:srp_code>/view-srp-requests/",
        view=general.srp_link_view_requests,
        name="view_srp_requests",
    ),
    path(
        route="srp-link/<str:srp_code>/enable/",
        view=general.enable_srp_link,
        name="enable_srp_link",
    ),
    path(
        route="srp-link/<str:srp_code>/disable/",
        view=general.disable_srp_link,
        name="disable_srp_link",
    ),
    path(
        route="srp-link/<str:srp_code>/delete/",
        view=general.delete_srp_link,
        name="delete_srp_link",
    ),
    path(
        route="srp-link/<str:srp_code>/request-srp/",
        view=general.request_srp,
        name="request_srp",
    ),
    path(
        route="srp-link/<str:srp_code>/complete/",
        view=general.complete_srp_link,
        name="complete_srp_link",
    ),
]



"""
General views
"""

# Django
from django.contrib import messages
from django.contrib.auth.decorators import permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponse
from django.shortcuts import redirect, render
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.authentication.decorators import permissions_required
from allianceauth.eveonline.models import EveCharacter
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag
from eveuniverse.models import EveType

# AA SRP
from aasrp import __title__
from aasrp.constants import KILLBOARD_DATA
from aasrp.form import (
    SrpLinkForm,
    SrpLinkUpdateForm,
    SrpRequestAcceptForm,
    SrpRequestAcceptRejectedForm,
    SrpRequestForm,
    SrpRequestRejectForm,
    UserSettingsForm,
)
from aasrp.helper.notification import notify_srp_team
from aasrp.helper.user import get_user_settings
from aasrp.models import Insurance, RequestComment, SrpLink, SrpRequest

logger = LoggerAddTag(my_logger=get_extension_logger(__name__), prefix=__title__)


@permission_required("aasrp.basic_access")
def srp_links(request: WSGIRequest, show_all_links: bool = False) -> HttpResponse:
    """
    SRP dashboard

    :param request:
    :type request:
    :param show_all_links:
    :type show_all_links:
    :return:
    :rtype:
    """

    logger_message = f"Dashboard with {'all' if show_all_links else 'available'} SRP links called by {request.user}"

    if show_all_links and not request.user.has_perm("aasrp.manage_srp"):
        messages.error(
            request=request,
            message=_("You do not have the needed permissions to view all SRP links"),
        )

        return redirect(to="aasrp:srp_links")

    logger.info(msg=logger_message)

    context = {"show_all_links": show_all_links}

    return render(
        request=request, template_name="aasrp/dashboard.html", context=context
    )


@permission_required("aasrp.basic_access")
def view_own_requests(request: WSGIRequest) -> HttpResponse:
    """
    View own SRP requests

    :param request:
    :type request:
    :return:
    :rtype:
    """

    logger.info(msg=f"Own SRP requests view called by {request.user}")

    return render(request=request, template_name="aasrp/view-own-requests.html")


@permission_required("aasrp.basic_access")
def user_settings(request: WSGIRequest) -> HttpResponse:
    """
    User settings

    :param request:
    :type request:
    :return:
    :rtype:
    """

    current_user_settings = get_user_settings(user=request.user)

    if request.method == "POST":
        user_settings_form = UserSettingsForm(
            data=request.POST, instance=current_user_settings
        )

        # If the form is valid, save the data to the database.
        if user_settings_form.is_valid():
            user_settings_form.save()

            messages.success(request, _("Settings saved."))

            return redirect("aasrp:user_settings")
    else:
        user_settings_form = UserSettingsForm(instance=current_user_settings)

    logger.info(f"User settings view called by {request.user}")

    context = {"user_settings_form": user_settings_form}

    return render(request, "aasrp/user-settings.html", context)


@permissions_required(("aasrp.manage_srp", "aasrp.create_srp"))
def srp_link_add(request: WSGIRequest) -> HttpResponse:
    """
    Add a SRP link

    :param request:
    :type request:
    :return:
    :rtype:
    """

    request_user = request.user

    logger.info(msg=f"Add SRP link form called by {request_user}")

    # If this is a POST request, we need to process the form data.
    if request.method == "POST":
        # Create a form instance and populate it with data from the request.
        form = SrpLinkForm(data=request.POST)

        # Check whether it's valid:
        if form.is_valid():
            srp_name = form.cleaned_data["srp_name"]
            fleet_time = form.cleaned_data["fleet_time"]
            fleet_type = form.cleaned_data["fleet_type"]
            fleet_doctrine = form.cleaned_data["fleet_doctrine"]
            aar_link = form.cleaned_data["aar_link"]

            srp_link = SrpLink(
                srp_name=srp_name,
                fleet_time=fleet_time,
                fleet_type=fleet_type,
                fleet_doctrine=fleet_doctrine,
                aar_link=aar_link,
                srp_code=get_random_string(length=16),
                fleet_commander=request.user.profile.main_character,
                creator=request.user,
            )
            srp_link.save()

            messages.success(
                request=request, message=_(f'SRP link "{srp_link.srp_code}" created')
            )

            return redirect(to="aasrp:srp_links")

    # If a GET (or any other method) we'll create a blank form.
    else:
        form = SrpLinkForm()

    context = {"form": form}

    return render(request=request, template_name="aasrp/link-add.html", context=context)


@permissions_required(("aasrp.manage_srp", "aasrp.create_srp"))
def srp_link_edit(request: WSGIRequest, srp_code: str) -> HttpResponse:
    """
    Add or edit AAR link

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :return:
    :rtype:
    """

    request_user = request.user

    logger.info(f"Edit SRP link form for SRP code {srp_code} called by {request_user}")

    # Check if the provided SRP code is valid
    try:
        srp_link = SrpLink.objects.get(srp_code=srp_code)
    except SrpLink.DoesNotExist:
        logger.error(
            msg=(
                f"Unable to locate SRP link using SRP code {srp_code} for "
                f"user {request_user}"
            )
        )

        messages.error(
            request=request,
            message=_(f"Unable to locate SRP link using SRP code {srp_code}"),
        )

        return redirect("aasrp:srp_links")

    # If this is a POST request, we need to process the form data.
    if request.method == "POST":
        form = SrpLinkUpdateForm(data=request.POST, instance=srp_link)

        if form.is_valid():
            srp_link.aar_link = form.cleaned_data["aar_link"]
            srp_link.save()

            messages.success(request, _("AAR link changed"))

            return redirect("aasrp:srp_links")
    else:
        form = SrpLinkUpdateForm(instance=srp_link)

    context = {"srp_code": srp_code, "form": form}

    return render(request, "aasrp/link-edit.html", context)


def _save_srp_request(  # pylint: disable=too-many-arguments, too-many-positional-arguments
    request: WSGIRequest,
    srp_link: SrpLink,
    killmail_link: str,
    ship_type_id: int,
    ship_value: str,
    victim_id: int,
    additional_info: str,
) -> SrpRequest:
    """
    Saving the SRP request

    :param request:
    :type request:
    :param srp_link:
    :type srp_link:
    :param killmail_link:
    :type killmail_link:
    :param ship_type_id:
    :type ship_type_id:
    :param ship_value:
    :type ship_value:
    :param victim_id:
    :type victim_id:
    :param additional_info:
    :type additional_info:
    :return:
    :rtype:
    """

    creator = request.user
    post_time = timezone.now()
    srp_request__character = EveCharacter.objects.get_character_by_id(
        character_id=victim_id
    )

    (
        srp_request__ship,
        created_from_esi,  # pylint: disable=unused-variable
    ) = EveType.objects.get_or_create_esi(id=ship_type_id)

    srp_request = SrpRequest.objects.create(
        killboard_link=killmail_link,
        creator=creator,
        srp_link=srp_link,
        character=srp_request__character,
        ship_name=srp_request__ship.name,
        ship=srp_request__ship,
        loss_amount=ship_value,
        post_time=post_time,
        request_code=get_random_string(length=16),
    )

    RequestComment.objects.bulk_create(
        [
            RequestComment(
                srp_request=srp_request,
                comment_type=RequestComment.Type.REQUEST_ADDED,
                creator=creator,
                new_status=SrpRequest.Status.PENDING,
            ),
            RequestComment(
                comment=additional_info,
                srp_request=srp_request,
                comment_type=RequestComment.Type.REQUEST_INFO,
                creator=creator,
            ),
        ]
    )

    insurance_information = SrpRequest.objects.get_insurance_for_ship_type(
        ship_type_id=ship_type_id
    )
    Insurance.objects.bulk_create(
        [
            Insurance(
                srp_request=srp_request,
                insurance_level=level["name"],
                insurance_cost=level["cost"],
                insurance_payout=level["payout"],
            )
            for level in insurance_information["levels"]
        ]
    )

    logger.info(
        msg=(
            f"Created SRP request on behalf of user {creator} "
            f"(character: {srp_request__character}) for fleet name {srp_link.srp_name} "
            f"with SRP code {srp_link.srp_code}"
        )
    )

    messages.success(
        request=request,
        message=_(f"Submitted SRP request for your {srp_request__ship.name}."),
    )

    return srp_request


@permission_required("aasrp.basic_access")
def request_srp(request: WSGIRequest, srp_code: str) -> HttpResponse:
    """
    SRP request

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :return:
    :rtype:
    """

    logger.info(
        msg=f"SRP request form for SRP code {srp_code} called by {request.user}"
    )

    # Check if the provided SRP code is valid
    try:
        srp_link = SrpLink.objects.get(srp_code=srp_code)
    except SrpLink.DoesNotExist:
        logger.error(
            msg=(
                f"Unable to locate SRP Fleet using SRP code {srp_code} for "
                f"user {request.user}"
            )
        )

        messages.error(
            request=request,
            message=_(f"Unable to locate SRP Fleet using SRP code {srp_code}"),
        )

        return redirect("aasrp:srp_links")

    # Check if the SRP link is still open
    if srp_link.srp_status != SrpLink.Status.ACTIVE:
        messages.error(
            request=request,
            message=_("This SRP link is no longer available for SRP requests."),
        )

        return redirect(to="aasrp:srp_links")

    # If this is a POST request, we need to process the form data.
    if request.method == "POST":
        # Create a form instance and populate it with data from the request.
        form = SrpRequestForm(data=request.POST)

        logger.debug(msg=f"Request type POST contains valid form: {form.is_valid()}")

        if form.is_valid():
            submitted_killmail_link = form.cleaned_data["killboard_link"]
            srp_request_additional_info = form.cleaned_data["additional_info"]

            # Parse killmail
            try:
                srp_kill_link_id = SrpRequest.objects.get_kill_id(
                    killboard_link=submitted_killmail_link
                )
                ship_type_id, ship_value, victim_id = SrpRequest.objects.get_kill_data(
                    kill_id=srp_kill_link_id
                )
            except ValueError as err:
                # Invalid killmail
                error_message_text = (
                    (
                        "Something went wrong, your kill mail "
                        f"({submitted_killmail_link}) could not be parsed: {str(err)}"
                    )
                    if str(err)
                    else (
                        f"Your kill mail link ({submitted_killmail_link}) is invalid "
                        "or the zKillboard API is not answering at the moment. "
                        "Please make sure you are using either "
                        f"{KILLBOARD_DATA['zKillboard']['base_url']}, "
                        f"{KILLBOARD_DATA['EveTools']['base_url']} "
                        f"or {KILLBOARD_DATA['EVE-KILL']['base_url']}"
                    )
                )

                messages.error(request=request, message=error_message_text)

                return redirect(to="aasrp:srp_links")

            if request.user.character_ownerships.filter(
                character__character_id=str(victim_id)
            ).exists():
                # Save the SRP request
                srp_request = _save_srp_request(
                    request=request,
                    srp_link=srp_link,
                    killmail_link=submitted_killmail_link,
                    ship_type_id=ship_type_id,
                    ship_value=ship_value,
                    victim_id=victim_id,
                    additional_info=srp_request_additional_info,
                )

                # Send a message to the srp team in their discord channel.
                notify_srp_team(
                    srp_request=srp_request, additional_info=srp_request_additional_info
                )

                return redirect(to="aasrp:srp_links")

            messages.error(
                request=request,
                message=_(
                    f"Character {victim_id} does not belong to your Auth account. Please add this character as an alt to your main and try again."  # pylint: disable=line-too-long
                ),
            )

            return redirect(to="aasrp:srp_links")

    # If a GET (or any other method) we'll create a blank form.
    else:
        logger.debug(msg=f"Returning blank SRP request form for {request.user}")
        form = SrpRequestForm()

    context = {"srp_link": srp_link, "form": form}

    return render(
        request=request, template_name="aasrp/request-srp.html", context=context
    )


@permission_required("aasrp.manage_srp")
def complete_srp_link(request: WSGIRequest, srp_code: str):
    """
    Mark an SRP link as completed

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :return:
    :rtype:
    """

    logger.info(
        f"Complete SRP link form for SRP link {srp_code} called by {request.user}"
    )

    try:
        srp_link = SrpLink.objects.get(srp_code=srp_code)
        srp_link.srp_status = SrpLink.Status.COMPLETED
        srp_link.save()

        messages.success(request, _("SRP link marked as completed"))
    except SrpLink.DoesNotExist:
        logger.error(
            f"Unable to locate SRP link using code {srp_code} for user {request.user}"
        )

        messages.error(request, _(f"Unable to locate SRP link with ID {srp_code}"))

    return redirect("aasrp:srp_links")


@permissions_required(("aasrp.manage_srp", "aasrp.manage_srp_requests"))
def srp_link_view_requests(request: WSGIRequest, srp_code: str) -> HttpResponse:
    """
    View SRP requests for a specific SRP code

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :return:
    :rtype:
    """

    logger.info(f"View SRP requests for SRP link {srp_code} called by {request.user}")

    # Check if the provided SRP code is valid
    try:
        srp_link = SrpLink.objects.get(srp_code=srp_code)
    except SrpLink.DoesNotExist:
        logger.error(
            f"Unable to locate SRP link using code {srp_code} for user {request.user}"
        )

        messages.error(request, _(f"Unable to locate SRP link with ID {srp_code}"))

        return redirect("aasrp:srp_links")

    context = {
        "srp_link": srp_link,
        "forms": {
            "reject_request": SrpRequestRejectForm(),
            "accept_request": SrpRequestAcceptForm(),
            "accept_rejected_request": SrpRequestAcceptRejectedForm(),
        },
    }

    return render(request, "aasrp/view-requests.html", context)


@permission_required("aasrp.manage_srp")
def enable_srp_link(request: WSGIRequest, srp_code: str):
    """
    Enable SRP link

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :return:
    :rtype:
    """

    logger.info(msg=f"Enable SRP link {srp_code} called by {request.user}")

    try:
        srp_link = SrpLink.objects.get(srp_code=srp_code)
        srp_link.srp_status = SrpLink.Status.ACTIVE
        srp_link.save()

        messages.success(
            request=request, message=_(f"SRP link {srp_code} (re-)activated.")
        )
    except SrpLink.DoesNotExist:
        logger.error(
            msg=f"Unable to locate SRP link using code {srp_code} for user {request.user}"
        )

        messages.error(
            request=request, message=_(f"Unable to locate SRP link with ID {srp_code}")
        )

    return redirect(to="aasrp:srp_links")


@permission_required("aasrp.manage_srp")
def disable_srp_link(request: WSGIRequest, srp_code: str):
    """
    Disable SRP link

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :return:
    :rtype:
    """

    logger.info(msg=f"Disable SRP link {srp_code} called by {request.user}")

    try:
        srp_link = SrpLink.objects.get(srp_code=srp_code)
        srp_link.srp_status = SrpLink.Status.CLOSED
        srp_link.save()

        messages.success(request=request, message=_(f"SRP link {srp_code} disabled."))
    except SrpLink.DoesNotExist:
        logger.error(
            msg=f"Unable to locate SRP link using code {srp_code} for user {request.user}"
        )

        messages.error(
            request=request, message=_(f"Unable to locate SRP link with ID {srp_code}")
        )

    return redirect(to="aasrp:srp_links")


@permission_required("aasrp.manage_srp")
def delete_srp_link(request: WSGIRequest, srp_code: str):
    """
    Delete SRP link

    :param request:
    :type request:
    :param srp_code:
    :type srp_code:
    :return:
    :rtype:
    """

    logger.info(msg=f"Delete SRP link {srp_code} called by {request.user}")

    try:
        srp_link = SrpLink.objects.get(srp_code=srp_code)
        srp_link.delete()

        messages.success(request=request, message=_(f"SRP link {srp_code} deleted."))
    except SrpLink.DoesNotExist:
        logger.error(
            msg=f"Unable to locate SRP link using code {srp_code} for user {request.user}"
        )

        messages.error(
            request=request, message=_(f"Unable to locate SRP link with ID {srp_code}")
        )

    return redirect(to="aasrp:srp_links")
